/*----------------------------------------------------------------*/
/* FrExtract.c             by B. Mours LAPP   Oct 22, 2002        */
/*                                                                */
/* This Matlab mex file extract from a frame file the data for    */ 
/*   one ADC channel (this is for Matlab version 5)               */
/*                                                                */
/* The input arguments are:                                       */
/*   1) file name                                                 */
/*   2) ADC or PROC name                                          */
/*   3) (optional) first frame (default= first frame in the file) */
/*   4) (optional) number of frame (default = 1 Frames)           */
/*                                                                */
/* Returned matlab data:                                          */
/*   1) ADC or PROC data (time serie)                             */
/*   2) (optional) time values relative to the first data point   */
/*   3) (optional) frequency values (for FFT's)                   */
/*   4) (optional) GPS starting time (in second.nanosec)          */
/*   5) (optional) starting time as a string                      */
/*   6) (optional) ADC comment as a string                        */
/*   7) (optional) ADC unit as a string                           */
/*   8) (optional) additional info: it is a 9 words vector which  */
/*       content:crate #, channel#, nBits#, biais, slope,         */
/*               sampleRate, timeOffset(S.N), fShift, overRange   */
/*      All this values are stored as double                      */
/*----------------------------------------------------------------*/
#include "mex.h"
#include "FrameL.h"

void  mexFunction( int nlhs,       mxArray *plhs[],
                   int nrhs, const mxArray *prhs[])
{FrFile *iFile;
 FrameH *frame;
 FrAdcData *adc;
 FrVect *vect;
 FrProcData *proc;
 int   status, first, nFrames;
 long buffSize, debugLevel, i, nBin, nData, index;
 double *data, dt, df, sampleRate;
 char *buff, *name, fileName[100],adcName[100], stringTime[30], 
      *comment, *units;
 time_t tp;

    /*--------------- check arguments ---------------------*/

 if(nrhs<2)
    {mexErrMsgTxt("Please provide the file name and ADC or PROC name as argument\n");
     return;}

 if(!mxIsChar(prhs[0]))
    {mexErrMsgTxt("The first arguement is not a filename\n");
     return;}
  
 if(!mxIsChar(prhs[1]))
    {mexErrMsgTxt("The second arguement is not the adc name \n");
     return;}
  
 if(nlhs==0)
    {mexErrMsgTxt("Please provide at least one output argument\n");
     return;}

 if(nrhs > 2)
     {data = mxGetPr(prhs[2]);
      first = data[0];}
 else{first = 0;}

 if(nrhs > 3)
     {data = mxGetPr(prhs[3]);
      nFrames = data[0];}
 else{nFrames = 1;}

    /*----------- open file --------------------------*/

  status=mxGetString(prhs[0],fileName,100);
  status=mxGetString(prhs[1],adcName,100);
  mexPrintf(" Open file:%s for channel:%s\n",fileName,adcName);
  mexPrintf(" first frame:%d nFrames=%d\n",first,nFrames);

  iFile = FrFileINew(fileName);
  if(iFile == NULL)
     {mexPrintf("Error during file opening\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return;}

     /*--------- read the first frame ------------------*/
   
  while((frame = FrameRead(iFile)) != NULL)
      {
	mexPrintf(" Frame %d run %d GPS : %d\n", 
             frame->frame, frame->run,frame->GTimeS);
       if(first == 0) break;
       if(frame->frame >= first) break; 
       FrameFree(frame);}

  if(frame == NULL) 
     {FrFileIEnd(iFile);  
      return;}

     /*------ find ADC and create output matrix ---------*/

  adc = FrAdcDataFind(frame,adcName);
  if(adc != NULL) 
    {vect       = adc->data;
     nBin       = adc->data->nData;
     sampleRate = adc->sampleRate;
     comment    = adc->comment;
     units      = adc->units;}
  else if((proc = FrProcDataFind(frame, adcName)) != NULL)
    {vect       = proc->data;
     nBin       = proc->data->nData;
     dt         = vect->dx[0];
     sampleRate = 1./vect->dx[0];
     comment    = proc->comment;
     units      = proc->data->unitY;}
   else 
     {FrFileIEnd(iFile);  
      FrameFree(frame);
      return;}

  nData = nBin*nFrames;
  dt = 1./sampleRate;
  df = sampleRate/((double)nData);
  plhs[0] = mxCreateDoubleMatrix(nData,1,mxREAL);
 
    /*------ fill time and frequency array ---------------*/

  if(nlhs > 1)
    {plhs[1] = mxCreateDoubleMatrix(nData,1,mxREAL);
     data = mxGetPr(plhs[1]);
     for(i=0; i<nData; i++)
        {data[i] = (double)i*dt;}}

  if(nlhs > 2)
    {plhs[2] = mxCreateDoubleMatrix(nData/2,1,mxREAL);
     data = mxGetPr(plhs[2]);
     for(i=0; i<nData/2; i++)
        {data[i] = (double)i*df;}}

  if(nlhs > 3)
    {plhs[3] = mxCreateDoubleMatrix(1,1,mxREAL);
     data = mxGetPr(plhs[3]);
     data[0] = (double)frame->GTimeS + 1.e-9*(double)frame->GTimeN;}

  if(nlhs > 4)
    {tp = frame->GTimeS + 315961200;
     strncpy(stringTime,ctime(&tp),24);
     name = mxCalloc(80,sizeof(char));
     sprintf(name,"Starting GPS time:%s (and %d usec)\n",
                            stringTime,frame->GTimeN/1000);
     plhs[4] = mxCreateString(name);
     mxFree(name);}	

  if(nlhs > 5)
    {if(comment != NULL)
         {name = mxCalloc(strlen(comment)+1,sizeof(char));
          strcpy(name, comment);
          plhs[5] = mxCreateString(name);
          mxFree(name);}		
     else{plhs[5] = mxCreateString("no comment");}}

  if(nlhs > 6)
    {if(units != NULL)
         {name = mxCalloc(strlen(units)+1,sizeof(char));
          strcpy(name, units);
          plhs[6] = mxCreateString(name);
          mxFree(name);}
     else{plhs[6] = mxCreateString("no unit");}}

  if(nlhs > 7)
    {plhs[7] = mxCreateDoubleMatrix(1,9,mxREAL);
     data = mxGetPr(plhs[7]);
     if(adc != NULL)
       {data[0] = (double) adc->channelGroup;
        data[1] = (double) adc->channelNumber;
        data[2] = (double) adc->nBits;
        data[3] = (double) adc->bias;
        data[4] = (double) adc->slope;
        data[5] = (double) sampleRate;
        data[6] = (double) adc->timeOffset;
        data[7] = (double) adc->fShift;
        data[8] = (double) adc->dataValid;
	vect = adc->data;}
     else
       {data[0] = (double) 0;
        data[1] = (double) 0;
        data[2] = (double) proc->data->wSize;
        data[3] = (double) 0;
        data[4] = (double) 0;
        data[5] = (double) sampleRate;
        data[6] = (double) proc->timeOffset;
        data[7] = (double) proc->fShift;
        data[8] = (double) 0;
	vect = proc->data;}}

    /*-------- copy data and read the next frames -------*/

  data = mxGetPr(plhs[0]);
  index = 0;
  while(1)
    {if(vect == NULL) 
        {mexPrintf(" no data for frame %ld\n",frame->frame);}
     else if(vect->type == FR_VECT_C)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->data[i];}}
     else if(vect->type == FR_VECT_2S)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataS[i];}}
     else if(vect->type == FR_VECT_4S)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataI[i];}}
     else if(vect->type == FR_VECT_1U)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataU[i];}}
     else if(vect->type == FR_VECT_2U)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataUS[i];}}
     else if(vect->type == FR_VECT_4U)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataUI[i];}}
     else if(vect->type == FR_VECT_4R)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataF[i];}}
     else if(vect->type == FR_VECT_8R)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataD[i];}}
     index += nBin;

     FrameFree(frame);
     frame = FrameRead(iFile);
     if(frame == NULL) break; 
     if(index >= nFrames*nBin) break; 

     vect = FrameGetV(frame,adcName);}
    
  if(frame != NULL) FrameFree(frame);

  FrFileIEnd(iFile);            /* Close file and release descriptor */

}
