/*---------------------------------------------------------------------------*/
/* File: FrFileChk.c                                               B. Mours  */
/*                                            Last update:     July 6, 2001  */
/*                                                                           */
/*  This program reads and check a frame file. In case of success it returns */
/*  a positive value which is the numberof frame in the file or a negative   */
/*  value in case of read (-1) or checksum error (-2).                       */
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "FrameL.h"

void Help()
{
 printf(" \n" 
     " This program reads and check all the frames in the file             \n"
     "-------------------------------------------------------------------- \n"
     " Syntax is: FrFileCheck  -i <input file>                             \n"
     "         -d <debug level>                                            \n"
     "         -t to scan the file using only the TOC                      \n"
     "         -s to scan the file using only sequentiel read(TOC not used)\n"
     "         -f GPS time of the first frame to scan (default=0)          \n"
     "         -l GPS time of the last frame to scan (default : 999999999.)\n"
     "         -h to get this help                                         \n"
     "-------------------------------------------------------------------- \n"
     " Remarks:                                                            \n"
     "  By default, FrCheck do first a sequentiel read to check the file   \n"
     "  checksum, then do a random access read to check the frame checksum.\n" 
     "  -i : Argument should be only one file names                        \n"
     "  -f and -t option works only when using the TOC                     \n"
     "-------------------------------------------------------------------- \n"
     "\n");
}

struct Parameters{
  char *name;                   /* input file name                     */
  int debug;                    /* debug level                         */
  double ftime;                 /* time for the first frame            */
  double ltime;                 /* time for the last frame             */
  FRBOOL seqRead;               /* if YES, perform a sequentiel read   */
  FRBOOL ranRead;               /* if YES, perform a random read(TOC)  */
};

void ReadParameters(int argc, char **argv);

struct Parameters Par;

/*-----------------------------------------------------------------scanFile--*/
int scanFile(FrFile *file)
/*---------------------------------------------------------------------------*/
{FrameH *frame = NULL;
 int nFrame = 0, nPrint = 1;
 
 if (Par.debug >0) printf("\n  Sequentiel file reading. Please wait....\n");
 file->chkSumFiFlag = FR_YES;

 while((frame = FrameReadRecycle(file, frame)) != NULL) 
   {nFrame++;
    if(nFrame != nPrint) continue;
    nPrint *= 2;
    if(Par.debug) printf("%9d frames read\n",nFrame);}
 
 if(Par.debug > 0) printf("   %d frame(s) in file.",nFrame);
 if(file->error != FR_OK)
      {printf(" Read Error. Last errors are:\n%s",FrErrorGetHistory());
       return(-1);}
 else {if(Par.debug > 0) printf(" No read error.");}
  
 if(file->chkTypeFiRead == 0)
   {if(Par.debug > 0) printf(" This file does not have a file checksum\n");}
 else
   {if(file->chkSumFiRead == file->chkSumFi)
     {if(Par.debug > 0) printf(" File Checksum OK (%x)\n",file->chkSumFiRead);}
    else
     {if(Par.debug > 0) printf(" File Cheksum error:\n       read = %x\n"
                                                 "   computed = %x\n",
                                file->chkSumFiRead, file->chkSumFi);}
    return(-2);}
 
 return(nFrame);
}

/*-----------------------------------------------------------------seekFile--*/
int seekFile(FrFile *file)
/*---------------------------------------------------------------------------*/
{FrameH *frame = NULL;
 int nFrame = 0, nPrint = 1;
 double gTimeS;

 if (Par.debug >0) printf("\n  Reading using the TOC. Please wait....\n");
 file->chkSumFrFlag = FR_YES;

 if(Par.debug > 0)
    printf("   Requested time range: %9.0f - %9.0f\n",Par.ftime, Par.ltime);

 gTimeS = FrFileITStart(file);
 if(gTimeS < 0) 
   {if(Par.debug > 0) printf(" No TOC available\n");
    return(0);}
 if(Par.debug > 0) printf("        File time range: %9.0f - %9.0f\n",
         gTimeS, FrFileITEnd(file));
 if(gTimeS < Par.ftime) gTimeS = Par.ftime;

 while((frame = FrameReadT(file, gTimeS)) != NULL) 
   {nFrame++;
    if(nFrame == nPrint)
      {nPrint *= 2;
       if(Par.debug) printf("%9d frames read\n",nFrame);}
    gTimeS = FrFileITNextFrame(file, gTimeS);
    if(gTimeS >= Par.ltime || gTimeS < 0.) break;
    FrameFree(frame);}
 
 if(Par.debug > 0) printf("   %d frame(s) in file.",nFrame);
 if(file->error != FR_OK)
      {printf(" Read Error. Last errors are:\n%s",FrErrorGetHistory());
       return(-1);}
 else {if(Par.debug > 0) printf(" No read error.");}
  
 if(Par.debug > 0) 
   {if(file->chkTypeFrRead == 0)
         {printf(" This file does not have frame checksums\n");}
    else {printf(" Frame Checksums OK\n");}}
 
 return(nFrame);
}

/*--------------------------------------------------------- Main ------------*/
int main(int argc, char **argv)
/*---------------------------------------------------------------------------*/
{FrFile *file;
 int nFrame = 0;
 
           /*----------------- Read input arguments -------------------------*/

 ReadParameters(argc, argv);

           /*---------------------------- Open the input file(s) ------------*/

 FrLibSetLvl(Par.debug-1);
 file = FrFileINew(Par.name);
 if(file == NULL)
    {printf("Input file (%s) open error:\n %s", Par.name, FrErrorGetHistory());
     return(-1);}

           /*----------------Start the Main loop ----------------------------*/

 if(Par.seqRead == FR_YES) nFrame = scanFile(file);
 if(Par.ranRead == FR_YES) nFrame = seekFile(file);
 
 exit(nFrame);
}


/*------------------------------------------------------ SetParameters ------*/
void ReadParameters(int argc, char **argv)
/*---------------------------------------------------------------------------*/
{int i;

 if (argc == 1)
    {Help();
     exit(0);}

 Par.name    = NULL;
 Par.debug   = 1; 
 Par.ftime   = 0;
 Par.ltime   = 999999999;
 Par.seqRead = FR_YES;
 Par.ranRead = FR_YES;

           /*------------------- Loop over the parameters -------------------*/

 for (i=1; i<argc; i++)
    {if     (strcmp(argv[i],"-h") == 0) 
       {Help(); 
        exit(0);}
    else if (strcmp(argv[i],"-i") == 0) {FrStrCpy(&(Par.name),argv[i+1]);}
    else if (strcmp(argv[i],"-d") == 0) {Par.debug = atoi(argv[i+1]);}
    else if (strcmp(argv[i],"-t") == 0) {Par.seqRead = FR_NO;}
    else if (strcmp(argv[i],"-s") == 0) {Par.ranRead = FR_NO;}
    else if (strcmp(argv[i],"-f") == 0) {Par.ftime = atof(argv[i+1]);}
    else if (strcmp(argv[i],"-l") == 0) {Par.ltime = atof(argv[i+1]);}}

           /*-------------------------- Check parameters---------------------*/

 if(Par.name == NULL) 
      {printf(" Please provide at least one input file\n");
       Help();
       exit(0);}

 if (Par.debug >0) printf(" Checking file %s\n", Par.name);

 return;
}
